generate

Function generate 

pub fn generate(args: GenerateArgs) -> Result<(), Vec<String>>
Generate Rust code for migrations and queries, also saves schema type information for use by queries.

Returns

  • Error - a list of validation or generation errors that occurred