pub fn expr_field_eq(param_name: impl Into<String>, f: &FieldHandle) -> ExprExpand description
Generates an expression checking for equality of a field and a parameter and the same type.
pub fn expr_field_eq(param_name: impl Into<String>, f: &FieldHandle) -> ExprGenerates an expression checking for equality of a field and a parameter and the same type.