pub fn field_param(param_name: impl Into<String>, f: &FieldHandle) -> ExprExpand description
Generates a param matching a field in name in type
pub fn field_param(param_name: impl Into<String>, f: &FieldHandle) -> ExprGenerates a param matching a field in name in type