set_field

Function set_field 

pub fn set_field(
    param_name: impl Into<String>,
    f: &FieldHandle,
) -> (FieldHandle, Expr)
Generates a field element for instert and update statements, to set a field from a parameter of the same type.