pub struct Query {
pub name: String,
pub body: Box<dyn QueryBody>,
pub res_count: QueryResCount,
pub res_name: Option<String>,
}Expand description
This represents an SQL query. A function will be generated which accepts a db
connection and query parameters, and returns the query results. Call the
new_* functions to get a builder.
Fields
name: Stringbody: Box<dyn QueryBody>res_count: QueryResCountres_name: Option<String>Auto Trait Implementations
impl Freeze for Query
impl !RefUnwindSafe for Query
impl !Send for Query
impl !Sync for Query
impl Unpin for Query
impl UnsafeUnpin for Query
impl !UnwindSafe for Query